﻿/* FmfileVersion.ahk ====== UTF-8 ========================================================================
              Get the Version of the FM/Book/MIF file
Location      E:\_DDDprojects\Get_FM_version,
Documents     www.daube.ch/docu/fmaker??.html
Comment       FM identifies document files with an initial sequence:
              <MakerFile 14.0>
              <MakerFile 11.0>          E:\FM-Specials\FM-11-tests\Gugus.mif
              <MIFFile 2018>            E:\FM-Specials\FM-15-tests\Tests+testfiles\EmptyPage.mif
              <BookFile 15.0>           E:\FM-Specials\FM-15-tests\Tests+testfiles\Gugus-book.book
              <MakerFile 4.0K>          E:\FM-Specials\FM-04-comments\FM-UserGroups.fm
              <Makerfile 15.0>, <BookFile 15.0> but <MIFFile 2015>

Usage         CompiledScript.exe [Switches] [Script Parameters]
              Be awaare of blanks in path names - use " in the Registry command
              FMfileVersion.exe "E:\FM-Specials\FM-13-tests\Support-Thai-enhanced\Thai Test FrameMaker.mif"
REGistry file This must be derived from an existing installaiton of
AHK version   1.1.26 up
              Assure that AutoHotkeySC.bin is the same as  Unicode 32-bit.bin
              Only this way the compiled scripts run on both x64 and x32 systems
Reference     https://www.online-tech-tips.com/computer-tips/windows-right-click-c...
Copyleft      2020 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2020-03-11 Start of development
              2020-03-16 published in fmaker85.html
*/ ; // ===========================================================================================
#Singleinstance Force
Global sTitle
     , sFileName
     , edition

; --- Check call of program --------------------------------
sTitle := UserMessage(0)
If (A_args.Length() < 1) {
  MsgBox , 16, %sTitle%, % UserMessage(1)
  ExitApp
}
; --- Read heder bytes from file ---------------------------
sFileName := A_args[1]
FileRead, sContent, *m32 %sFileName%    ; get the initial 32 bytes
If (ErrorLevel) {
  MsgBox , 16, %sTitle%, % UserMessage(2, sFileName)
  ExitApp
}
; --- Analyse the file contents ----------------------------
iPos := RegExMatch(sContent, "(\d+).*>", fmVersion)
If (iPos == 0) {
  MsgBox , 16, %sTitle%, % UserMessage(3, sFileName)
  ExitApp
}
; --- Finally we can say something about the file ----------
fmVersion := SubStr(fmVersion, 1, StrLen(fmVersion) - 1)
isCharacter := SubStr(fmVersion, 0, 1)  ; FM versiona < 10 used letters to indicate build, e.g. 5.0Y
If (Instr("ABCDEFGHIJKLMNOPQRSTUVWXYZ", isCharacter) > 0 ) {
  fmVersion := SubStr(fmVersion, 1, StrLen(fmVersion) - 1)
}
edition := "-"
If (fmVersion > 12) {
  Switch fmVersion
  {
  Case 13: edition := "2015"
  Case 14: edition := "2017"
  Case 15: edition := "2019"
  Case 16: edition := "2020"
  Case 17: edition := "2022"
  }
} Else If (fmVersion > 50) {
  edition := fmVersion
  Switch edition
  {
  Case 2015: fmVersion := "13"
  Case 2017: fmVersion := "14"
  Case 2018: fmVersion := "15"
  Case 2019: fmVersion := "15"
  Case 2020: fmVersion := "16"
  Case 2022: fmVersion := "17"
  }
}
; --- Finally the message to the user ----------------------
Gui, Font, s12, Verdana
Gui, Add, Text, , % UserMessage(4, sFileName, fmVersion, edition)
Gui, Show , , %sTitle%

; User must quit the message via X-button

; ============ functions ==========================================================================
UserMessage(j, p1:="???", p2:="???", p3:="???", p4:="???", p5:="???") {
; Arguments   j         number of message to be returned
;             p1 … p5   parameters for the messages
; Returns     messge[j]
  RegRead, sCode, HKEY_CURRENT_USER\Control Panel\International, LocaleName
  sLanguage := SubStr(sCode, 1, 2)   ; de-CH → de (any German)
; see  ISO 639-1 codes
; sLanguage := "fr"                     ; Test fr version
  msg := Array()
  Switch sLanguage {
    Case "de":
      sTitle := "FM Version aus Datei ermitteln"
      msg[1] := "Dieses Programm erfordert einen Dateipfad als Parameter."
      msg[2] := "Lesefehler für Datei" . p1 . "»."
      msg[3] := "Datei`n" . p1 . "`nist keine FrameMaker Datei. Der Dateityp kann z.B. durch Hochladen zu http://checkfiletype.com/ ermittelt werden."
      msg[4] := "Datei " . p1 . "`n`nöffnet mit FrameMaker " . p2 . " (Ausgabe " . p3 . ")"
; Tranlated by https://www.deepl.com/translator
    Case "es":                          ; Spanish
      sTitle := "Obtener la versión FM del archivo"
      msg[1] := "Esta aplicación debe ser llamada con un parámetro: el nombre del archivo a examinar."
      msg[2] := "Error de lectura para el archivo " . p1 . "»."
      msg[3] := "Archivo`n" . p1 . "`nno es un archivo de FrameMaker.`nPuede que quieras determinar su tipo subiéndolo a http://checkfiletype.com/"
      msg[4] := "Archivo " . p1 . "`n`nse abre con FrameMaker " . p2 . " (Edición " . p3 . ")"
    Case  "fr":
      sTitle := "Obtenir la version FM du fichier"
      msg[1] := "Cette demande doit être appelée avec un paramètre: le nom du fichier à examiner."
      msg[2] := "Erreur de lecture pour le fichier «" . p1 . "»."
      msg[3] := "Le fichier`n" . p1 . "`nn'est pas un fichier FrameMaker, mais vous pouvez déterminer son type en le téléchargeant sur http://checkfiletype.com/"
      msg[4] := "Le fichier " . p1 . "`n`ns'ouvre avec FrameMaker " . p2 . " (Édition " . p3 . ")"
    Case "it":
      sTitolo := "Ottieni la versione FM da file"
      msg[1] := "Questa applicazione deve essere chiamata con un parametro: el nome del file da esaminare."
      msg[2] := "Errore di lettura per il file «" . sFileName . "»."
      msg[3] := "File`n" . p1 . "`nnon è un file FrameMaker.`nSi può determinare il suo tipo caricandolo su http://checkfiletype.com/."
      msg[4] := "File" . p1 . "`n'n si apre con FrameMaker " . p2 . " (Edizione " . p3 . ")"
    Case "ja":                          ; Japanese
      sTitle := "ファイルからFMバージョンを取得"
      msg[1] := "このアプリケーションは、パラメータ: 検査対象のファイル名を指定して呼び出す必要があります。"
      msg[2] := "ファイル «" . sFileName . "»."
      msg[3] := "File`n" .p1 . "`nは FrameMaker ファイルではありません。`nhttp://checkfiletype.com/ にアップロードしてファイルの種類を確認してください。"
      msg[4] := "ファイル" . "`n`nは FrameMaker で開く " . p2 . " (エディション" . p3 . ")"
    Case "nl":                          ; Dutch
      sTitle := "FM-versie uit bestand ophalen."
      msg[1] := "Deze applicatie moet worden aangeroepen met een parameter: de te onderzoeken bestandsnaam."
      msg[2] := "Leesfout voor bestand «" . sFileName . "»."
      msg[3] := "Bestand `n" . p1 . "n is geen FrameMaker-bestand.`nU kunt het type ervan bepalen door het te uploaden naar http://checkfiletype.com/."
      msg[4] := "Bestand " . p1 . "`n`nopent met FrameMaker " . p2 . " (Editie " . p3 . ")"
    Case "pl":                          ; Polish
      sTitle := "Pobierz wersję FM z pliku"
      msg[1] := "Ta aplikacja musi być wywołana z parametrem: azwa pliku do zbadania."
      msg[2] := "Błąd odczytu dla pliku «" . sFileName . "»."
      msg[3] := "Plik`n" . p1 . "`nnie jest plikiem programu FrameMaker.`nMożesz określić jego typ, przesyłając go na adres http://checkfiletype.com/"
      msg[4] := "Plik " . p1 . "`n`notwiera się z FrameMaker " . p2 . " (Edycja " . p3 . ")"
    Case "pt":                          ; Portuguese
      sTitle := "Obter Versão FM a partir de Arquivo"
      msg[1] := "Esta aplicação deve ser chamada com um parâmetro: nome do arquivo a ser examinado."
      msg[2] := "Read error for file «" . sFileName . "»."
      msg[3] := "File`n" . p1 . "`nnão é um arquivo FrameMaker.`nVocê pode querer determinar seu tipo, carregando-o para http://checkfiletype.com/."
      msg[4] := "Arquivo" . p1 . "n`n abre com FrameMaker " . p2 . " (Edição " . p3 . ")"
    Case "ru":                          ; Russian
      Название := "Получить FM-версию из файла."
      msg[1] := "Это приложение должно быть вызвано с параметром: не имя файла, который будет рассматриваться."
      msg[2] := "Ошибка чтения для файла «" . sFileName . "»."
      msg[3] := "Файл`n" . p1 . "`nне является файлом FrameMaker.`nВы можете определить его тип, загрузив его на http://checkfiletype.com/."
      msg[4] := "Файл" . p1 . "`n`n` открывается с FrameMaker" . p2 . " (редакция " . p3 . ")"
; Tranlated by https://translate.google.com/
    Case "da":                          ; Danish, Dansk
       sTitle := "Hent FM-version fra fil"
       msg[1] := "Denne applikation skal kaldes med en parameter: filnavnet, der skal undersøges."
       msg[2] := "Læs fejl for fil «" . sFileName . "» ."
       msg[3] := "File`n" . p1 . "` n er ikke en FrameMaker-fil.`nDu ønsker muligvis at bestemme dens type ved at uploade den til http://checkfiletype.com/ "
       msg[4] := "Fil" . p1 . "` n `n åbnes med FrameMaker" . p2 . "(Udgave" . p3 . ")"
    Case "el":                          ; Greek
       sTitle := "Λήψη έκδοσης FM από αρχείο"
       msg[1] := "Αυτή η εφαρμογή πρέπει να καλείται με μια παράμετρο: το όνομα του αρχείου που θα εξεταστεί."
       msg[2] := "Σφάλμα ανάγνωσης για αρχείο «" . sFileName . "»."
       msg[3] := "Αρχείο" . p1 . "Το`n δεν είναι ένα αρχείο FrameMaker. '' Μπορεί να θέλετε να προσδιορίσετε τον τύπο του μεταφορτώνοντάς τον στο http://checkfiletype.com/ "
       msg[4] := "Αρχείο" . p1 . "`n`nανοίγει με FrameMaker " . p2 . "(Έκδοση" . p3 . ")"
   Case "fi":                           ; Finnish, Suomi
       sTitle := "Hae FM-versio tiedostosta"
       msg[1] := "Tätä sovellusta on kutsuttava parametrilla:tutkittavan tiedoston nimi."
       msg[2] := "Luku virhe virhe tiedostolle «" . sFileName . "»."
       msg[3] := "Tiedosto`n" . p1 . "`n ei ole FrameMaker-tiedosto. `nVoit ehkä määrittää sen tyypin lähettämällä sen osoitteeseen http://checkfiletype.com/."
       msg[4] := "Tiedosto" . p1 . "`n`n aukeaa FrameMakerilla " . p2 . "(Painos" . p3 . ")"
    Case "is":                          ; Icelandic
       sTitle := "Fáðu FM útgáfu af skrá"
       msgstr [1] := "Þessa forrit verður að kalla með breytu:heiti skjalsins sem á að skoða. "
       msg[2] := "Lesa villu fyrir skrána «" . sFileName . "». "
       msg[3] := "File`n" . p1 . "`n er ekki FrameMaker skrá. `Þú gætir viljað ákvarða gerð þess með því að hlaða henni inn á http://checkfiletype.com/"
       msg[4] := "Skrá" . p1 . "`n`nopnar með FrameMaker" . p2 . "(Útgáfa" . p3 . ")"
    Case "no":                          ; Norwegian (NYNorsk ?
       sTitle := "Hent FM-versjon fra fil"
       msg[1] := "Denne applikasjonen må kalles med en parameter: filnavnet som skal undersøkes."
       msg[2] := "Lesefeil for filen «" . sFileName . "»."
       msg[3] := "File`n" . p1 . "`n er ikke en FrameMaker-fil. `Det kan være lurt å bestemme hvilken type det er ved å laste den opp til http://checkfiletype.com/"
       msg[4] := "Fil" . p1 . "`n`n åpnes med FrameMaker" . p2 . "(Utgave" . p3 . ")"
    Case "sv":                          ; Swedish
       sTitle := "Hämta FM-version från fil"
       msg[1] := "Den här applikationen måste anropas med en parameter: filnamnet som ska undersökas. "
       msg[2] := "Läs fel för fil «" . sFileName . "»."
       msg[3] := "File`n" . pl . "`n är inte en FrameMaker-fil. `Du kanske vill bestämma sin typ genom att ladda upp den till http://checkfiletype.com/"
       msg[4] := "Fil" . pl . "`n`n öppnar med FrameMaker" . p2 . "(Utgåva" . p3 . ")"
; My own English
    Default:
      sTitle := "Get FM Version from File"
      msg[1] := "This application must be called with a parameter: the file name to be examined."
      msg[2] := "Read error for file «" . sFileName . "»."
      msg[3] := "File`n" . p1 . "`nis not a FrameMaker file.`nYou may wish to determine it's type by uploading it to http://checkfiletype.com/"
      msg[4] := "File " . p1 . "`n`nopens with FrameMaker " . p2 . " (Edition " . p3 . ")"
  }
  If (j == 0) {
    return sTitle
  } Else {
    return msg[j]
  }
} ; --- end UserMessage
